# -*- Mode: Python -*-
# -*- coding: ascii -*-

"""
Instances Based on File
"""

import sys
import math
import lwsdk
import random

__author__     = "Oliver Hotz"
__date__       = "June 2015"
__copyright__  = "Copyright (C) 2015 Origami Digital"
__version__    = "1.0"
__maintainer__ = "Oliver Hotz"
__email__      = "oliver@origamidigital.com"
__status__     = "Utility"
__lwver__      = "11.5"

RESOURCE = '\04(k:"%s" c:LWPy)'

# Define an exception for a Mesh Info failure
class BadMeshInfo(Exception):
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)
       
class OD_FileInstance(lwsdk.IInstancer):
 
  def __init__(self, context):
    super(OD_FileInstance, self).__init__()

    self.ui                 = None 
    self.panel              = None 
    self._time              = 0.0
    self._frame             = 0
    self.count              = 0
    self.generated          = -1
    self.InstancesID        = []
    self.InstancesNum       = []
    self.InstancesPos       = []
    self.InstancesRot       = []
    self.InstancesScl       = []
    self.file               = ""

    # #Query Mesh Points and their PTag
    # self._meshinfo = lwsdk.LWObjectInfo().meshInfo( lwsdk.LWInterfaceInfo().selected_items()[0], True )
    # iterator = self._meshinfo.createMeshIterator( lwsdk.LWMESHITER_POINT )
    # count = 0
    # while count != int( lwsdk.LWObjectInfo().numPoints( lwsdk.LWInterfaceInfo().selected_items()[0] ) ):
      # pointID =  self._meshinfo.iterateMesh( iterator )
      # pos = self._meshinfo.polTag(pointID, 1)
      # print "Tag", pos
      # count += 1
    
    
  #Deleting Instance
  def __del__(self):
    pass

  # LWInterfaceFuncs ------------------------------------
  def inter_ui(self):
    if not self.panel:
      if not self.ui:
        self.ui    = lwsdk.LWPanels      ()
        self.panel = self.ui.create      ('OD Filebased Instance Generator  ' + chr(0xa9) + '2015 by Oliver Hotz @ Origami Digital, LLC')
        self.c1    = self.panel.load_ctl ("Instance Data Filename:", 80);
        self.c1.set_str                  (self.file)
    
        if self.panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
          print "Aborted..."
          self.ui.destroy(self.panel)
          return lwsdk.AFUNC_OK # when cancel was clicked
    
    if self.generated == -1:
      #File format:
      #loop of:
      #First line: full path of to be instanced object
      #second line: amount of instances for this object
      #after second line, until next ascii line, is the pos;rot (in degrees);scl transformation for each instance
      f = open(self.c1.get_str(), "r")
      lines = f.readlines()
      f.close()
      for x in range(0, len(lines)):
        if lines[x][0] == "\"":
          obj = lines[x].replace("\n", "").strip().replace("\"", "")
          #load source object
          lwsdk.command("LoadObject " + obj)
          self.InstancesID.append(lwsdk.itemid_to_str(lwsdk.LWInterfaceInfo().selected_items()[0]))
          numInstances = lines[x+1].replace("\n", "").strip()
          self.InstancesNum.append(numInstances)
          for l in range(x+2, x+2+int(numInstances)):
            pos = lines[l].replace("\n", "").strip().split(";")[0]
            rot = lines[l].replace("\n", "").strip().split(";")[1]
            scl = lines[l].replace("\n", "").strip().split(";")[2]
            self.InstancesPos.append(pos)
            self.InstancesRot.append(rot)
            self.InstancesScl.append(scl)
      self.generated = 0
    
    
    return lwsdk.AFUNC_OK

  def panel_close(self, panel, data):
    self.panel = 0
    pass

  # LWRenderFuncs ---------------------------------------
  def rend_init(self, mode):
    pass

  def rend_cleanup(self):
    pass

  def rend_newTime(self, frame, time):
    pass

  # LWInstancer -----------------------------------------
  def evaluate(self, ia):
    #this condition is run when instances first need to be generated
    if self.generated == 0:
      #first loop for ever "instanced object"
      for x in range(0, len(self.InstancesID)):
        print "Instances for Object:", lwsdk.LWItemInfo().name(lwsdk.str_to_itemid(self.InstancesID[x])), "Amount:", int(self.InstancesNum[x])
        #second loop for each amount of instances per instanced object
        for i in range(0, int(self.InstancesNum[x])):
          #getting pos/rot/scl for the instance transforms
          pos = lwsdk.Vector(float(self.InstancesPos[self.count].split(",")[0]),float(self.InstancesPos[self.count].split(",")[1]),float(self.InstancesPos[self.count].split(",")[2]))
          rot = lwsdk.Vector(math.radians(float(self.InstancesRot[self.count].split(",")[0])),math.radians(float(self.InstancesRot[self.count].split(",")[1])),math.radians(float(self.InstancesRot[self.count].split(",")[2])))
          scl = lwsdk.Vector(float(self.InstancesScl[self.count].split(",")[0]),float(self.InstancesScl[self.count].split(",")[1]),float(self.InstancesScl[self.count].split(",")[2]))
          #generating Instance ID
          inst_id = lwsdk.LWItemInstancerFuncs().createInstance(ia.instancer)
          #attaching item to instance
          lwsdk.LWItemInstancerFuncs().setInstance(inst_id, self.count, lwsdk.str_to_itemid(self.InstancesID[x]), 1)
          #set transform of item for instance
          lwsdk.LWItemInstancerFuncs().setMotionStep(inst_id, 0, pos, scl, rot)
          #red color
          self.color = 16711680
          #setting display to bounding box mode
          lwsdk.LWItemInstancerFuncs().setInstanceDrawer(inst_id, 6)
          #setting color to red
          lwsdk.LWItemInstancerFuncs().setInstanceDrawerColor(inst_id, self.color)
          self.count = self.count + 1
          #update Gui
          lwsdk.LWInstUpdate(self)
          
      self.generated = 1 #so that we only generate the instances once, otherwise they will be continuously generated.

            
  # LWInstanceFuncs -------------------------------------
  def inst_copy(self, source):
    return None     # LWError

  def inst_descln(self):
    if self.InstancesID != "":
      return "OD_FileInstance: Count ::: " + str(self.count)
    else:
      return "OD_FileInstance None"
      
  def inst_acquire(self):
    # here you would 'acquire' any resources that would need to be
    # explicitly released (in the inst_release() method) later.
    pass

  def inst_save(self, state):
    #save file path for instance data file
    state.lwsave_str(str(self.file))

  def inst_load(self, state):
    #load file path for instance data file
    result, val = state.lwload_str()
    if result == 0:
      return "File Path Error"
    self.file = val
    
  def inst_release(self):
    # release any resources acquired earlier.  these would include
    # resourcese that impose reference counts on our class instance
    # (e.g., a non-modal LWPanels instance).

    # shut down the requester, if it is open

    if self.panel:
      self.ui.destroy(self.panel)
      self.panel = None
      self.ui = None

  # LWItemFuncs  ----------------------------------------
  def item_changeID(self, itemid_list):
    pass

ServerTagInfo = [
                ( "OD FileInstance", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                ( "OD FileInstance", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH ),
                ]

ServerRecord = { lwsdk.InstancerFactory("OD_FileInstance", OD_FileInstance) : ServerTagInfo }